@@ -33,7 +33,7 @@ from . import WrapMode
 from .. import coredata
 from ..mesonlib import quiet_git, GIT, ProgressBar, MesonException
 from ..interpreterbase import FeatureNew
-from  .. import mesonlib
+from .. import mesonlib
 
 if T.TYPE_CHECKING:
     import http.client
@@ -151,7 +151,6 @@ class PackageDefinition:
             if f'{what}_filename' in self.values and f'{what}_url' not in self.values:
                 FeatureNew(f'Local wrap patch files without {what}_url', '0.55.0').use(self.subproject)
 
-
     def parse_wrap_section(self, config: configparser.ConfigParser) -> None:
         if len(config.sections()) < 1:
             raise WrapException(f'Missing sections in {self.basename}')
@@ -419,7 +418,7 @@ class Resolver:
         if is_shallow and self.is_git_full_commit_id(revno):
             # git doesn't support directly cloning shallowly for commits,
             # so we follow https://stackoverflow.com/a/43136160
-            verbose_git(['init', '-b', 'meson-dummy-branch', self.directory], self.subdir_root, check=True)
+            verbose_git(['-c', 'init.defaultBranch=meson-dummy-branch', 'init', self.directory], self.subdir_root, check=True)
             verbose_git(['remote', 'add', 'origin', self.wrap.get('url')], self.dirname, check=True)
             revno = self.wrap.get('revision')
             verbose_git(['fetch', *depth_option, 'origin', revno], self.dirname, check=True)
@@ -557,8 +556,7 @@ class Resolver:
                 mlog.log('Using', mlog.bold(self.packagename), what, 'from cache.')
                 return cache_path
 
-            if not os.path.isdir(self.cachedir):
-                os.mkdir(self.cachedir)
+            os.makedirs(self.cachedir, exist_ok=True)
             self.download(what, cache_path)
             return cache_path
         else:
